/*
  Copyright (C) 2007 Crawford Currie http://wikiring.com and Arthur Clemens
  Copyright (C) 2009 Martin Cleaver http://www.blendedperspectives.com
  All Rights Reserved.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version. For
  more details read LICENSE in the root of the TWiki distribution.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  As per the GPL, removal of this notice is prohibited.
*/
/* 
2009-01-04: Martin Cleaver: made thread safe as part of Foswiki:Items8032

*/

/*
Notes:
   * This class would benefit from use of jQuery instead of raw HTTP calls. MC
*/

// The TWikiTiny class object
var TWikiTiny = {

    twikiVars : null,
    metaTags : null,

    tml2html : new Array(), // callbacks, attached in plugins
    html2tml : new Array(), // callbacks, attached in plugins

    // Get a TWiki variable from the set passed
    getTWikiVar : function (name) {
        if (TWikiTiny.twikiVars == null) {
            var sets = tinyMCE.getParam("twiki_vars", "");
            TWikiTiny.twikiVars = eval(sets);
        }
        return TWikiTiny.twikiVars[name];
    },

    expandVariables : function(url) {
        for (var i in TWikiTiny.twikiVars) {
            url = url.replace('%' + i + '%', TWikiTiny.twikiVars[i], 'g');
        }
        return url;
    },

    enableSave: function(enabled) {
        var status = enabled ? null : "disabled";
        var elm = document.getElementById("save");
        if (elm) {
            elm.disabled = status;
        }
        elm = document.getElementById("preview");
        if (elm) {
            elm.style.display = 'none'; // Item5263: broken preview
            elm.disabled = status;
        }
    },


    reqObject: function(method, text) {
        // Work out the REST URL+PATH from the handler
        var url = TWikiTiny.getTWikiVar("SCRIPTURL");
        var suffix = TWikiTiny.getTWikiVar("SCRIPTSUFFIX");
        if (suffix == null) suffix = '';

        url += "/rest" + suffix + "/WysiwygPlugin/" + method;
        var path = TWikiTiny.getTWikiVar("WEB") + '.' + TWikiTiny.getTWikiVar("TOPIC");

	// Set up the object
	var req = null;
        if (tinyMCE.isIE) {
            // branch for IE/Windows ActiveX version
            req = new ActiveXObject("Microsoft.XMLHTTP");
        } else {
            // branch for native XMLHttpRequest object
            req = new XMLHttpRequest();
        }

        req.open("POST", url, true);
        req.setRequestHeader(
            "Content-type", "application/x-www-form-urlencoded");
        var params = "nocache=" + encodeURIComponent((new Date()).getTime())
        + "&topic=" + encodeURIComponent(path)
        + "&text=" + encodeURIComponent(text);
    
        req.setRequestHeader(
            "Content-length", params.length);
        req.setRequestHeader("Connection", "close");
	return {req: req, params: params};
    },

    transform2: function(closure, method, inputMarkup, onReadyToSend, onReply, onFail) {

	// TODO: ASSERT closure is not null

//	alert("Transforming "+closure.editor_id + " with method "+method);

        closure.request = new Object();
	var reqAndParams =  TWikiTiny.reqObject(method, inputMarkup);

	closure.request.req = reqAndParams.req;
	var params = reqAndParams.params;

        closure.request.req.onreadystatechange = function() {
            // Callback for XMLHttpRequest
            // only if request.req shows "complete"
            if (closure.request.req.readyState == 4) {
                // only if "OK"
                if (closure.request.req.status == 200) {
	//	alert("Reply for "+closure.editor_id);
                    onReply();
                } else {
                    onFail();
                }
            }
        };
        onReadyToSend();
        closure.request.req.send(params);
    },


    initialisedFromServer : new Array(),

    // Set up content for the initial edit
    setUpContent : function(editor_id, body, doc) {
        // If we haven't done it before, then transform from TML
        // to HTML. We need this test so that pressing the 'back'
        // button from a failed save doesn't banjax the old content.
        if (TWikiTiny.initialisedFromServer[editor_id]) {return} 
        var editor = tinyMCE.getInstanceById(editor_id);
        TWikiTiny.switchToWYSIWYG(editor);
        TWikiTiny.initialisedFromServer[editor_id] = true;
    },

    // Convert HTML content to textarea. Called from the WYSIWYG->raw switch
    switchToRaw : function (editor) {
        // As shown by OliverKrueger in Item5138, trivial text may include
        // UTF-8 chars. These need to be encoded to entities before we can
        // pass the string back to the server. This is done in triggerSave,
        // but note that it requires cleanup:true to work.
        editor.triggerSave(false, true);
        var text = editor.oldTargetElement.value;

	var closure = new Object;  // closure needed to persist the editor and request context for 
	closure.editor = editor;
	closure.editor_id = editor.editorId;
	closure.request = null;


        // Evaluate post-processors
        for (var i = 0; i < TWikiTiny.html2tml.length; i++) {
            var cb = TWikiTiny.html2tml[i];
            text = cb.apply(editor, [ editor, text ]);
        }
        TWikiTiny.transform2(
            closure, "html2tml", text,
            function () {
                // onReadyToSend
                TWikiTiny.enableSave(false);
                var te = closure.editor.oldTargetElement;
                te.value = "Please wait... retrieving page from server";
            },
            function () {
                var te = closure.editor.oldTargetElement;
                var text = closure.request.req.responseText;
                te.value = text;
                TWikiTiny.enableSave(true);
            },
            function () {
                var te = closure.editor.oldTargetElement;
                te.value = "There was a problem retrieving the page: "
                    + closure.request.req.statusText;
                //TWikiTiny.enableSave(true); leave it disabled
            });

	TWikiTiny.conversionButton("show", editor);

        // SMELL: what if there is already an onchange handler?
        editor.oldTargetElement.onchange = function() {
            var editor = tinyMCE.getEditoranceById(eid);
            editor.isNotDirty = false;
            return true;
        }
    },

    // Convert textarea content to HTML. This is invoked from the content
    // setup handler, and also from the raw->WYSIWYG switch
    switchToWYSIWYG : function (editor) {
	var closure = new Object;  // needed for the closure
	closure.editor = editor;
	closure.editor_id = IFRAME_ID;
	closure.request = null;
        // Kill the change handler to avoid excess fires
        editor.oldTargetElement.onchange = null;
        // Need to tinyMCE.execCommand("mceToggleEditor", null, editor_id);
        TWikiTiny.transform2(
            closure, "tml2html", editor.oldTargetElement.value,
            function () {
                // onReadyToSend
                TWikiTiny.enableSave(false);

                tinyMCE.setInnerHTML(
                    closure.editor.getBody(),
                    "<span class='twikiAlert'>Please wait... retrieving page from server.</span>");
            },
            function () {
                // Handle the reply
                var text = closure.request.req.responseText;

                // Evaluate any registered pre-processors
                for (var i = 0; i < TWikiTiny.tml2html.length; i++) {
                    var cb = TWikiTiny.tml2html[i];
                    text = cb.apply(editor, [ editor, text ]);
                }
                tinyMCE.setInnerHTML(closure.editor.getBody(), text);
                closure.editor.isNotDirty = true;
                TWikiTiny.enableSave(true);
            },
            function () {
                // Handle a failure
                tinyMCE.setInnerHTML(
                    closure.editor.getBody(),
                    "<div class='twikiAlert'>"
                    + "There was a problem retrieving the page: "
                    + closure.request.req.statusText + "</div>");
                //TWikiTiny.enableSave(true); leave save disabled
            });

	TWikiTiny.conversionButton("hide", editor);
    },

   conversionButton: function(showOrHide, editor) {
        // Show or Hide the button for the switch back to WYSIWYG mode
        var eid = editor.editorId;
        var id = eid + "_2WYSIWYG";
        var el = document.getElementById(id);

	if (showOrHide == "hide") {
	        // Hide the conversion button, if it exists
	        if (el) {	
        	    // exists, hide it
	            el.style.display = "none";
	        }
        } {
	        if (el) {
        	    // exists, unhide it
	            el.style.display = "block";
        	} else {
	            // does not exist, create it
	            el = document.createElement('INPUT');
	            el.id = id;
	            el.type = "button";
	            el.value = "WYSIWYG";
	            el.className = "twikiButton";
	            el.onclick = function () {
	                tinyMCE.execCommand("mceToggleEditor", null, editor.editorId);
	                return false;
        	    }
		// SMELL: does this need placing at XX1
	            // Need to insert after to avoid knackering 'back'
	            var pel = editor.oldTargetElement.parentNode;
	            pel.insertBefore(el, editor.oldTargetElement);
        	}
	}
	// XX1 SMELL ref.
    },

    // Callback on save. Make sure the WYSIWYG flag ID is there.
    saveCallback : function(editor_id, html, body) {
        // Evaluate any registered post-processors
        var editor = tinyMCE.getInstanceById(editor_id);
        for (var i = 0; i < TWikiTiny.html2tml.length; i++) {
            var cb = TWikiTiny.html2tml[i];
            html = cb.apply(editor, [ editor, html ]);
        }
        var secret_id = tinyMCE.getParam('twiki_secret_id');
        if (secret_id != null && html.indexOf(
                '<!--' + secret_id + '-->') == -1) {
            // Something ate the ID. Probably IE. Add it back.
            html = '<!--' + secret_id + '-->' + html;
        }
        return html;
    },

    // Called 
    // Called on URL insertion, but not on image sources. Expand TWiki
    // variables in the url.
    convertLink : function(url, node, onSave){
        if(onSave == null)
            onSave = false;
        var orig = url;
        var pubUrl = TWikiTiny.getTWikiVar("PUBURL");
        var vsu = TWikiTiny.getTWikiVar("VIEWSCRIPTURL");
        url = TWikiTiny.expandVariables(url);
        if (onSave) {
            if ((url.indexOf(pubUrl + '/') != 0) &&
                (url.indexOf(vsu + '/') == 0)) {
                url = url.substr(vsu.length + 1);
                url = url.replace(/\/+/g, '.');
                if (url.indexOf(TWikiTiny.getTWikiVar('WEB') + '.') == 0) {
                    url = url.substr(TWikiTiny.getTWikiVar('WEB').length + 1);
                }
            }
        } else {
            if (url.indexOf('/') == -1) {
                // if it's a wikiword, make a suitable link
                var match = /^((?:\w+\.)*)(\w+)$/.exec(url);
                if (match != null) {
                    var web = match[1];
                    var topic = match[2];
                    if (web == null || web.length == 0) {
                        web = TWikiTiny.getTWikiVar("WEB");
                    }
                    web = web.replace(/\.+/g, '/');
                    web = web.replace(/\/+$/, '');
                    url = vsu + '/' + web + '/' + topic;
                }
            }
        }
        return url;
    },

    // Called from Insert Image, when the image is inserted. The resultant
    // URL is only used when displaying the image in the picture dialog. It
    // is thrown away (reverts to the typed address) when the image is
    // actually inserted, at which time convertLink is called.
    convertPubURL : function(url){
        var orig = url;
        var base = TWikiTiny.getTWikiVar("PUBURL") + '/'
        + TWikiTiny.getTWikiVar("WEB") + '/'
        + TWikiTiny.getTWikiVar("TOPIC") + '/';
        url = TWikiTiny.expandVariables(url);
        if (url.indexOf('/') == -1) {
            url = base + url;
        }
        return url;
    },

    getMetaTag : function(inKey) {
        if (TWikiTiny.metaTags == null || TWikiTiny.metaTags.length == 0) {
            // Do this the brute-force way because of the Firefox problem
            // seen sporadically on Bugs where the DOM appears complete, but
            // the META tags are not all found by getElementsByTagName
            var head = document.getElementsByTagName("META");
            head = head[0].parentNode.childNodes;
            TWikiTiny.metaTags = new Array();
            for (var i = 0; i < head.length; i++) {
                if (head[i].tagName != null &&
                    head[i].tagName.toUpperCase() == 'META') {
                    TWikiTiny.metaTags[head[i].name] = head[i].content;
                }
            }
        }
        return TWikiTiny.metaTags[inKey]; 
    },
    
    install : function() {
        // find the TINYMCEPLUGIN_INIT META
        var tmce_init = TWikiTiny.getMetaTag('TINYMCEPLUGIN_INIT');
        if (tmce_init != null) {
            eval("tinyMCE.init({" + unescape(tmce_init) + "});");
            return;
        }
        alert("Unable to install TinyMCE; <META name='TINYMCEPLUGIN_INIT' is missing"); 
    }
};
